/**
 * Class for representing ranges of time, analogous to Python's
 * datetime.timedelta (http://docs.python.org/lib/datetime-timedelta.html).
 *
 * Constructor accepts strings of the form "<days>D<hours>:<min>:<sec>".
 * Valid examples:
 *  one hour: '1:00:00'
 *  one day: '1D0:00:00'
 *  one day, two hours: '1D2:00:00'
 *  one week: '7D0:00:00'
 *  negative one week and three minutes: '-7D0:03:00'
 */
class com.stamen.timespan.Delta
{
    private var seconds:Number;

    public function Delta()
    {
        if(arguments.length == 1) {
            if(typeof arguments[0] == typeof '') {
                parse(arguments[0]);
                
            } else {
                throw new Error('Illegal argument to Delta(): "'+arguments[0]+'" (type: '+(typeof arguments[0])+')');
            }
        }
    }
    
    private function parse(str:String)
    {
        var polarity:Number = 1;
        
        if(str.charAt(0) == '-') {
            polarity = -1;
            str = str.slice(1);
        }
        
        var majorParts:Array = str.split('D');
        
        if(majorParts.length == 2) {
            seconds = parseInt(majorParts.shift().toString()) * 86400;
        }
        
        if(majorParts.length == 1) {
        
            var minorParts:Array = majorParts[0].split(':');
            
            for(var factor:Number = 1; minorParts.length; factor *= 60)
                seconds += parseInt(minorParts.pop().toString()) * factor;
            
        } else {
            throw new Error('Illegal argument to parse(): "'+str+'"');

        }
        
        seconds *= polarity;
    }
    
    public function toString():String
    {
        var prefix:String = (seconds < 0) ? '-' : '';

        var secs:Number = Math.abs(seconds) % 86400;
        var days:Number = (Math.abs(seconds) - secs) / 86400;
        
        var time:Array = [];
        var part:Number;
        
        for(var factor:Number = 1; secs; factor *= 60) {
            part = (secs % (factor * 60)) / factor;
            secs -= part * factor;
        
            time.unshift((part < 10) ? '0'+part : part);
        }
        
        return prefix + days + 'D' + time.join(':');
    }
    
    public function toMilliseconds():Number
    {
        return seconds * 1000;
    }
}