/**
 * Class for representing a point in time, analogous to built-in Date class.
 * Arguments to constructor must be in a limited RFC 3339 form for UTC times.
 *
 * Constructor accepts strings of the form "<year>-<month>-<day>T<hours>:<min>:<sec>Z".
 */
class com.stamen.timespan.Point extends Date
{
    public function Point()
    {
        if(arguments.length == 1) {
            if(typeof arguments[0] == typeof '') {
                parse(arguments[0]);
                
            } else if(arguments[0] instanceof Date) {
                setTime(arguments[0].getTime());

            } else {
                throw new Error('Illegal argument to Point(): "'+arguments[0]+'" (type: '+(typeof arguments[0])+')');
            }
        }
    }

    private function parse(str:String):Void
    {
        // e.g. 1990-12-31T23:59:60Z
        
        if(str.length != 20)
            throw new Error('Point.parse(): Wrong timestamp length');
        
        if(str.charAt(10) != 'T')
            throw new Error('Point.parse(): No "T" at char 10, "'+str.charAt(10)+'" instead.');
        
        if(str.charAt(19) != 'Z')
            throw new Error('Point.parse(): No "Z" at char 19, "'+str.charAt(19)+'" instead.');
            
        var yr:Number = parseInt(str.slice(0, 4));
        var mo:Number = parseInt(str.slice(5, 7)) - 1;
        var da:Number = parseInt(str.slice(8, 10));
        var hr:Number = parseInt(str.slice(11, 13));
        var mi:Number = parseInt(str.slice(14, 16));
        var sc:Number = parseInt(str.slice(17, 19));
        
        setTime(Date.UTC(yr, mo, da, hr, mi, sc));
    }
    
    public function toString():String
    {
        // e.g. 1990-12-31T23:59:60Z
        
        var yr:String = (getUTCYear()+1900).toString();
        var mo:String = ((getUTCMonth()+1) >= 10) ? (getUTCMonth()+1).toString() : '0'+(getUTCMonth()+1).toString();
        var da:String = (getUTCDate()      >= 10) ? getUTCDate().toString()      : '0'+getUTCDate().toString();
        var hr:String = (getUTCHours()     >= 10) ? getUTCHours().toString()     : '0'+getUTCHours().toString();
        var mi:String = (getUTCMinutes()   >= 10) ? getUTCMinutes().toString()   : '0'+getUTCMinutes().toString();
        var sc:String = (getUTCSeconds()   >= 10) ? getUTCSeconds().toString()   : '0'+getUTCSeconds().toString();
        
        return [yr, mo, da].join('-') + 'T' + [hr, mi, sc].join(':') + 'Z';
    }
    
    public function toDate():Date
    {
        return new Date(getYear() + 1900, getMonth(), getDate(), getHours(), getMinutes(), getSeconds(), getMilliseconds());
    }
}