import com.stamen.timespan.*;

/**
 * Class for representing a span in time, between two points in time.
 *
 * Constructor accepts strings of the following forms:
 *  Point: "<year>-<month>-<day>T<hours>:<min>:<sec>Z"
 *  Point/Point: "<year>-<month>-<day>T<hours>:<min>:<sec>Z/<year>-<month>-<day>T<hours>:<min>:<sec>Z"
 *  Point/Delta: "<year>-<month>-<day>T<hours>:<min>:<sec>Z/<days>D<hours>:<min>:<sec>"
 *
 * See Point and Delta constructors for more details.
 */
class com.stamen.timespan.Range
{
    private static var STYLE_POINT:Number = 0;
    private static var STYLE_SEGMENT:Number = 1;
    private static var STYLE_VECTOR:Number = 2;

    private var point:Point;
    private var delta:Delta;
    private var end:Point;
    private var style:Number;
    
    public var startDate:Date;
    public var duration:Number;
    public var endDate:Date;

    public function Range(str:String)
    {
        var parts:Array = str.split('/');
        
        if(parts.length == 2) {
            point = new Point(parts[0]);
            
            try {
                end = new Point(parts[1]);
                style = STYLE_SEGMENT;
            } catch(e:Error) {
                delta = new Delta(parts[1]);
                style = STYLE_VECTOR;
            }
        
        } else if(parts.length == 1) {
            point = new Point(parts[0]);
            style = STYLE_POINT;
        
        } else {
            throw new Error('Illegal argument to Range(): "'+str+'"');
        
        }
        
        startDate = getStartDate();
        duration = getDuration();
        endDate = getEndDate();
    }
    
    public function copy():Range
    {
        return new Range(toString());
    }
    
    public function toString():String
    {
        if(style == STYLE_POINT) {
            return point.toString();
            
        } else if(style == STYLE_SEGMENT) {
            return point.toString() + '/' + end.toString();
            
        } else if(style == STYLE_VECTOR) {
            return point.toString() + '/' + delta.toString();

        } else {
            throw new Error('Illegal style: '+style);

        }
    }
    
    private function getStartDate():Date
    {
        if(style == STYLE_POINT) {
            return point.toDate();
            
        } else if(style == STYLE_SEGMENT) {
            if(point.toDate().getTime() <= end.toDate().getTime()) {
                return point.toDate();
                
            } else {
                return end.toDate();
                
            }
            
        } else if(style == STYLE_VECTOR) {
            if(delta.toMilliseconds() >= 0) {
                return point.toDate();
                
            } else {
                var d:Date = new Date();
                d.setTime(point.toDate().getTime() + delta.toMilliseconds());
                return d;
                
            }
        }
    }
    
    private function getEndDate():Date
    {
        if(style == STYLE_POINT) {
            return point.toDate();
            
        } else if(style == STYLE_SEGMENT) {
            if(point.toDate().getTime() <= end.toDate().getTime()) {
                return end.toDate();
                
            } else {
                return point.toDate();
                
            }
            
        } else if(style == STYLE_VECTOR) {
            if(delta.toMilliseconds() >= 0) {
                var d:Date = new Date();
                d.setTime(point.toDate().getTime() + delta.toMilliseconds());
                return d;
                
            } else {
                return point.toDate();
                
            }
        }
    }
    
    private function getDuration():Number
    {
        if(style == STYLE_POINT) {
            return 0;
            
        } else if(style == STYLE_SEGMENT) {
            return Math.abs(point.toDate().getTime() - end.toDate().getTime());
            
        } else if(style == STYLE_VECTOR) {
            return Math.abs(delta.toMilliseconds());

        }
    }
}